@@ -12,10 +12,11 @@ from .cwt import (
 )
 from .encrypted_cose_key import EncryptedCOSEKey
 from .exceptions import CWTError, DecodeError, EncodeError, VerifyError
+from .helpers.hcert import load_pem_hcert_dsc
 from .recipient import Recipient
 from .signer import Signer
 
-__version__ = "1.0.0"
+__version__ = "1.3.0"
 __title__ = "cwt"
 __description__ = "A Python implementation of CWT/COSE"
 __url__ = "https://python-cwt.readthedocs.io"
@@ -31,6 +32,7 @@ __all__ = [
     "encode_and_sign",
     "encode_and_encrypt",
     "decode",
+    "load_pem_hcert_dsc",
     "set_private_claim_names",
     "CWT",
     "COSE",
