@@ -42,7 +42,9 @@ class CWT(CBORProcessor):
             raise ValueError("leeway should be positive number.")
         self._leeway = leeway
 
-        self._cose = COSE(kid_auto_inclusion=True, alg_auto_inclusion=True)
+        self._cose = COSE(
+            kid_auto_inclusion=True, alg_auto_inclusion=True, verify_kid=True
+        )
         self._claim_names: Dict[str, int] = {}
 
     @classmethod
@@ -78,6 +80,13 @@ class CWT(CBORProcessor):
         """
         return self._expires_in
 
+    @expires_in.setter
+    def expires_in(self, expires_in: int):
+        if expires_in <= 0:
+            raise ValueError("expires_in should be positive number.")
+        self._expires_in = expires_in
+        return
+
     @property
     def leeway(self) -> int:
         """
@@ -85,6 +94,20 @@ class CWT(CBORProcessor):
         """
         return self._leeway
 
+    @leeway.setter
+    def leeway(self, leeway: int):
+        if leeway <= 0:
+            raise ValueError("leeway should be positive number.")
+        self._leeway = leeway
+        return
+
+    @property
+    def cose(self) -> COSE:
+        """
+        The underlying COSE object.
+        """
+        return self._cose
+
     def encode(
         self,
         claims: Union[Claims, Dict[str, Any], Dict[int, Any], bytes],
