@@ -18,7 +18,7 @@ from .recipient_algs.direct_key import DirectKey
 from .recipient_algs.ecdh_aes_key_wrap import ECDH_AESKeyWrap
 from .recipient_algs.ecdh_direct_hkdf import ECDH_DirectHKDF
 from .recipient_interface import RecipientInterface
-from .utils import to_cose_header
+from .utils import parse_apu, parse_apv, to_cose_header
 
 
 class Recipient:
@@ -88,28 +88,23 @@ class Recipient:
         else:
             recipient = data
 
-        # salt
-        if "salt" in recipient:
-            if not isinstance(recipient["salt"], str):
-                raise ValueError("salt should be str.")
-            unprotected[-20] = recipient["salt"].encode("utf-8")
-
         # alg
         sender_key = None
-        if "alg" in recipient:
-            if not isinstance(recipient["alg"], str):
-                raise ValueError("alg should be str.")
-            if recipient["alg"] not in COSE_ALGORITHMS_RECIPIENT:
-                raise ValueError(f"Unsupported or unknown alg: {recipient['alg']}.")
-            if recipient["alg"] == "direct":
-                unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
-            elif recipient["alg"] in COSE_ALGORITHMS_KEY_WRAP:
-                unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
-                sender_key = COSEKey.from_jwk(recipient)
-            else:
-                protected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
-            if recipient["alg"] in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT.keys():
-                sender_key = COSEKey.from_jwk(recipient)
+        if "alg" not in recipient:
+            raise ValueError("alg should be specified.")
+        if not isinstance(recipient["alg"], str):
+            raise ValueError("alg should be str.")
+        if recipient["alg"] not in COSE_ALGORITHMS_RECIPIENT:
+            raise ValueError(f"Unsupported or unknown alg: {recipient['alg']}.")
+        if recipient["alg"] == "direct":
+            unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
+        elif recipient["alg"] in COSE_ALGORITHMS_KEY_WRAP:
+            unprotected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
+            sender_key = COSEKey.from_jwk(recipient)
+        else:
+            protected[1] = COSE_ALGORITHMS_RECIPIENT[recipient["alg"]]
+        if recipient["alg"] in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT.keys():
+            sender_key = COSEKey.from_jwk(recipient)
 
         # kid
         if "kid" in recipient:
@@ -117,6 +112,31 @@ class Recipient:
                 raise ValueError("kid should be str.")
             unprotected[4] = recipient["kid"].encode("utf-8")
 
+        # salt
+        if "salt" in recipient:
+            if not isinstance(recipient["salt"], str):
+                raise ValueError("salt should be str.")
+            unprotected[-20] = recipient["salt"].encode("utf-8")
+
+        # context
+        if "context" in recipient:
+            if not isinstance(recipient["context"], dict):
+                raise ValueError("context should be dict.")
+            apu = parse_apu(recipient["context"])
+            if apu[0]:
+                unprotected[-21] = apu[0]
+            if apu[1]:
+                unprotected[-22] = apu[1]
+            if apu[2]:
+                unprotected[-23] = apu[2]
+            apv = parse_apv(recipient["context"])
+            if apv[0]:
+                unprotected[-24] = apv[0]
+            if apv[1]:
+                unprotected[-25] = apv[1]
+            if apv[2]:
+                unprotected[-26] = apv[2]
+
         return cls.new(protected, unprotected, sender_key=sender_key)
 
     @classmethod
