@@ -59,7 +59,7 @@ class TestRecipientInterface:
         assert r.unprotected == {}
         assert r.ciphertext == b""
         assert isinstance(r.recipients, list)
-        assert r.kid is None
+        assert r.kid == b""
         assert r.alg == 0
         assert len(r.recipients) == 0
         with pytest.raises(NotImplementedError):
@@ -160,6 +160,13 @@ class TestRecipientInterface:
                 [RecipientInterface()],
                 "unprotected[4](kid) should be bytes.",
             ),
+            (
+                {4: "our-secret"},
+                {},
+                b"",
+                [RecipientInterface()],
+                "protected[4](kid) should be bytes.",
+            ),
             (
                 {1: "alg-a"},
                 {4: b"our-secret"},
@@ -237,6 +244,106 @@ class TestRecipient:
         assert isinstance(recipient, RecipientInterface)
         assert recipient.alg == -3
 
+    def test_recipient_from_jwk_with_context(self):
+        recipient = Recipient.from_jwk(
+            {
+                "kty": "oct",
+                "alg": "direct+HKDF-SHA-256",
+                "context": {
+                    "apu": {
+                        "id": "sender-01",
+                        "nonce": "xxx",
+                        "other": "yyy",
+                    },
+                    "apv": {
+                        "id": "recipient-01",
+                        "nonce": "abc",
+                        "other": "def",
+                    },
+                },
+            }
+        )
+        assert isinstance(recipient, RecipientInterface)
+        assert recipient.alg == -10
+        assert recipient._unprotected[-21] == b"sender-01"
+        assert recipient._unprotected[-22] == b"xxx"
+        assert recipient._unprotected[-23] == b"yyy"
+        assert recipient._unprotected[-24] == b"recipient-01"
+        assert recipient._unprotected[-25] == b"abc"
+        assert recipient._unprotected[-26] == b"def"
+
+    def test_recipient_from_jwk_with_context_id(self):
+        recipient = Recipient.from_jwk(
+            {
+                "kty": "oct",
+                "alg": "direct+HKDF-SHA-256",
+                "context": {
+                    "apu": {
+                        "id": "sender-01",
+                    },
+                    "apv": {
+                        "id": "recipient-01",
+                    },
+                },
+            }
+        )
+        assert isinstance(recipient, RecipientInterface)
+        assert recipient.alg == -10
+        assert recipient._unprotected[-21] == b"sender-01"
+        assert -22 not in recipient._unprotected
+        assert -23 not in recipient._unprotected
+        assert recipient._unprotected[-24] == b"recipient-01"
+        assert -25 not in recipient._unprotected
+        assert -26 not in recipient._unprotected
+
+    def test_recipient_from_jwk_with_context_nonce(self):
+        recipient = Recipient.from_jwk(
+            {
+                "kty": "oct",
+                "alg": "direct+HKDF-SHA-256",
+                "context": {
+                    "apu": {
+                        "nonce": "xxx",
+                    },
+                    "apv": {
+                        "nonce": "abc",
+                    },
+                },
+            }
+        )
+        assert isinstance(recipient, RecipientInterface)
+        assert recipient.alg == -10
+        assert -21 not in recipient._unprotected
+        assert recipient._unprotected[-22] == b"xxx"
+        assert -23 not in recipient._unprotected
+        assert -24 not in recipient._unprotected
+        assert recipient._unprotected[-25] == b"abc"
+        assert -26 not in recipient._unprotected
+
+    def test_recipient_from_jwk_with_context_other(self):
+        recipient = Recipient.from_jwk(
+            {
+                "kty": "oct",
+                "alg": "direct+HKDF-SHA-256",
+                "context": {
+                    "apu": {
+                        "other": "yyy",
+                    },
+                    "apv": {
+                        "other": "def",
+                    },
+                },
+            }
+        )
+        assert isinstance(recipient, RecipientInterface)
+        assert recipient.alg == -10
+        assert -21 not in recipient._unprotected
+        assert -22 not in recipient._unprotected
+        assert recipient._unprotected[-23] == b"yyy"
+        assert -24 not in recipient._unprotected
+        assert -25 not in recipient._unprotected
+        assert recipient._unprotected[-26] == b"def"
+
     @pytest.mark.parametrize(
         "data, msg",
         [
@@ -253,11 +360,15 @@ class TestRecipient:
                 "alg should be str.",
             ),
             (
-                {"kid": 123},
+                {"alg": "direct", "kid": 123},
                 "kid should be str.",
             ),
             (
-                {"salt": 123},
+                {"kty": "oct", "alg": "A128KW", "kid": 123},
+                "kid should be str.",
+            ),
+            (
+                {"kty": "oct", "alg": "A128KW", "salt": 123},
                 "salt should be str.",
             ),
             (
@@ -276,6 +387,10 @@ class TestRecipient:
                 {"kty": "oct", "alg": "A128KW", "k": 123},
                 "k should be str.",
             ),
+            (
+                {"kty": "oct", "alg": "direct+HKDF-SHA-256", "context": []},
+                "context should be dict.",
+            ),
         ],
     )
     def test_recipient_from_jwk_with_invalid_arg(self, data, msg):
@@ -304,42 +419,33 @@ class TestRecipients:
         assert key.alg == 4
         assert key.kid == b"our-secret"
 
-    def test_recipients_extract_with_empty_recipient(self):
-        key = COSEKey.from_symmetric_key(
-            "mysecret", alg="HMAC 256/64", kid="our-secret"
-        )
-        r = Recipients([RecipientInterface()])
-        with pytest.raises(ValueError) as err:
-            r.extract([key])
-            pytest.fail("extract() should fail.")
-        assert "Failed to derive a key." in str(err.value)
-
     def test_recipients_extract_without_key(self):
         r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
         with pytest.raises(ValueError) as err:
             r.extract([])
             pytest.fail("extract() should fail.")
-        assert "Failed to derive a key." in str(err.value)
+        assert "key is not found." in str(err.value)
 
     def test_recipients_extract_without_context(self, material):
         r = Recipients(
             [
-                RecipientInterface(
-                    unprotected={"alg": "direct+HKDF-SHA-256", "kid": "02"}
+                Recipient.new(
+                    unprotected={"alg": "direct+HKDF-SHA-256", "kid": "02"},
                 )
-            ]
+            ],
+            True,
         )
         with pytest.raises(ValueError) as err:
             r.extract(keys=[material])
             pytest.fail("extract() should fail.")
-        assert "Failed to derive a key." in str(err.value)
+        assert "context should be set." in str(err.value)
 
     def test_recipients_extract_with_empty_recipients(self, material, context):
         r = Recipients([])
         with pytest.raises(ValueError) as err:
             r.extract(context=context, keys=[material])
             pytest.fail("extract() should fail.")
-        assert "Failed to derive a key." in str(err.value)
+        assert "No recipients." in str(err.value)
 
     def test_recipients_extract_with_multiple_materials(self, material, context):
         r1 = Recipient.from_jwk(
@@ -411,7 +517,7 @@ class TestRecipients:
         with pytest.raises(ValueError) as err:
             r.extract([key])
             pytest.fail("extract() should fail.")
-        assert "Failed to derive a key." in str(err.value)
+        assert "key is not found." in str(err.value)
 
     def test_recipients_from_list(self):
         try:
